/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.lang.schema.internal;

import com.google.gson.ExclusionStrategy;
import com.google.gson.FieldAttributes;
import com.google.gson.GsonBuilder;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;

import com.redhat.thermostat.lang.schema.*;

import com.redhat.thermostat.lang.schema.annotations.Schema;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Service;

import java.io.IOException;

@Component
@Service(JSONService.class)
public class JSONServiceImpl implements JSONService {

    private GsonBuilder builder;

    public JSONServiceImpl() {
        this(false);
    }

    public JSONServiceImpl(boolean prettyPrinting) {


        builder = new GsonBuilder();
        if (prettyPrinting) {
            builder.setPrettyPrinting();
        }

        AutoAdapter autoAdapter = new AutoAdapter();

        builder.registerTypeAdapter(boolean.class, autoAdapter);
        builder.registerTypeAdapter(byte.class, autoAdapter);
        builder.registerTypeAdapter(short.class, autoAdapter);
        builder.registerTypeAdapter(int.class, autoAdapter);
        builder.registerTypeAdapter(long.class, autoAdapter);
        builder.registerTypeAdapter(double.class, autoAdapter);
        builder.registerTypeAdapter(float.class, autoAdapter);

        builder.registerTypeAdapter(Boolean.class, autoAdapter);
        builder.registerTypeAdapter(Byte.class, autoAdapter);
        builder.registerTypeAdapter(Short.class, autoAdapter);
        builder.registerTypeAdapter(Integer.class, autoAdapter);
        builder.registerTypeAdapter(Long.class, autoAdapter);
        builder.registerTypeAdapter(Double.class, autoAdapter);
        builder.registerTypeAdapter(Float.class, autoAdapter);

        builder.addSerializationExclusionStrategy(new AutoExclusionStategy());

    }

    @Override
    public String serialiase(Object target) {
        // TODO - consider registering this class with the schema registry
        return builder.create().toJson(target);
    }

    private class AutoAdapter extends TypeAdapter<Object> {

        @Override
        public void write(JsonWriter out, Object value) throws IOException {
            out.value("" + value);
        }

        @Override
        public Object read(JsonReader in) throws IOException {
            throw new UnsupportedOperationException("read() not yet implemented");
        }
    }

    public class AutoExclusionStategy implements ExclusionStrategy {

        public boolean shouldSkipClass(Class<?> arg0) {
            return false;
        }

        public boolean shouldSkipField(FieldAttributes field) {
            return (field.getAnnotation(Schema.class) == null);
        }
    }
}
